### UFCFVK-15-2 - Internet of things - Challenge 2

_A Morse Code communicator, written in C++ for the **BBC micro:bit**_

Send simple Morse Code messages from one _micro:bit_ to another over just one wire¹.

> ¹ two wires needed when the _micro:bit_s do not share a common ground

## Build Instructions

> _These instructions are intended for UNIX-like OSes, including GNU/Linux, macOS and friends. If you have access to a UNIX or POSIX environment on Windows such as MinGW or Cygwin, these instructions might also work there._

This project is written in C++ and uses the Device Abstraction Layer provided by Lancaster University.

You will need **git** to download the project code and **yotta** to build it.

Instructions for installing both of these and other prerequisites needed for building this project can be found [here](https://microbit-micropython.readthedocs.io/en/latest/devguide/installation.html#).

Once the prerequisites have been installed, in the root directory of the repo, run the following:

```sh
# set the compiler target architecture (only need to do once per shell session)
yt target bbc-microbit-classic-gcc
# this compiles the actual code
yt build
```

> Be patient during compilation, if this is the first time you have cloned the source code, first-time compilation may take a while.

When compilation has completed successfully, the executable file which is to be flashed onto the _micro:bit_s can be found at `build/bbc-microbit-classic-gcc/source/com-saxbophone-iot-coursework-challenge-2-combined.hex`

This needs to be flashed onto each _micro:bit_. This can be done by copying the executable file onto the removable drive that each _micro:bit_ exposes when connected by USB to a computer, for example:

```sh
# NOTE: this assumes that your micro:bit is mounted at "/media/sf_MICROBIT/"
#       it's probably actually located somewhere else on your machine
cp build/bbc-microbit-classic-gcc/source/com-saxbophone-iot-coursework-challenge-2.hex /media/sf_MICROBIT/
```

> **Don't forget** to flash _both_ of your _micro:bit_s!

Each _micro:bit_ will flash its LED when it is being flashed. If successful, the LED should turn solid and the program should start running.

## Setup Instructions

You will need at least one wire connected between one of the edge connector pins of your two _micro:bit_s for this software to communicate over. This is referred to as `Line` in the internal documentation of the program code.

If your _micro:bit_s do not share a common ground (such as if they are powered from different USB power supplies), you will need an additional wire connecting the ground pin of both _micro:bit_s together.

By default, this software communicates on Pin 1, so you should connect Pin 1 of each _micro:bit_ to the other's.

To change this, modify these macros found near the top of `source/main.cpp`:

```c++
#define COMMS_PIN_NUMBER 1
#define COMMS_PIN_NAME MICROBIT_PIN_P1
#define COMMS_PIN_EVENT_ID MICROBIT_ID_IO_P1
```

I used a school-style crocodile clip wire, about 25cm in length.

## Usage

When the _micro:bit_s power on, they will wait for pairing mode. Pairing mode has to be initiated by the user and the user must take care to follow some steps in a specific order and take note of the output on both _micro:bit_ displays to ensure correct pairing occurs. _micro:bit_s must be paired before communication can be made.

### Pairing Process

> Note: _micro:bit_s scroll some text at boot explaining visually which button is for choosing **Master** and which is for choosing **Slave**. This also happens after every failed pairing. You will need to wait for the text to disappear before the buttons will have any effect.

1. Pick a _micro:bit_ to be the _Slave_ —this is the _micro:bit_ that will be the receiver from a communications point of view. The other _micro:bit_ shall be the _Master_, which initiates communications, receives user input of messages and sends these messages to the _Slave_.
2. Once both _micro:bit_s have booted, scrolled some text and then blanked their displays:
	- **Press Button <kbd>B</kbd>** on the **Slave**
	- **Press Button <kbd>A</kbd>** on the **Master**
3. Wait for the _micro:bit_s to pair. Pairing takes about 3 seconds to complete.
	- If the chosen **Master** scrolls the text `PM` across its screen **and** the chosen **Slave** scrolls the text `PS` across its screen, then the _micro:bit_s have been successfully paired. Morse Code messages can now be sent from the **Master** to the **Slave**.
	- If **either** _micro:bit_ scrolls the text `PF`, then pairing has failed. Any _micro:bit_s which are aware of the failure will automatically re-prompt for pairing.
		> Note: If only one _micro:bit_ displays this failure message, but the other shows `PS`, then this _micro:bit_ must be manually reset by power-cycling the device or using the on-board reset button.

#### Pairing Error Codes

When any _micro:bit_ fails to pair, it displays a numeric error code number after displaying its role-specific failure text. This table explains the meaning of the different codes:

| Code | Master/Slave | Meaning                                                                                                                                                                     |
|------|--------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| -1   | N/A          | Invalid Role type (Master/Slave) supplied to pairing method (program logic or memory failure)                                                                               |
| 1    | Slave        | Configured IO Pin does not support reading in digital mode                                                                                                                  |
| 2    | Master       | Configured IO Pin does not support reading in digital mode, OR The micro:bit on the other end of the wire is also trying to be the Master                                   |
| 3    | Slave        | Timed out waiting for Master to start the pairing process                                                                                                                   |
| 4    | Master       | The micro:bit on the other end of the wire is also trying to be the Master                                                                                                  |
| 5    | Slave        | The Master didn't send a long enough pairing ping                                                                                                                           |
| 6    | Master       | Time out waiting for Slave to respond to the sent pairing ping                                                                                                              |
| 7    | Slave        | The Master sent a pairing ping that was too long                                                                                                                            |
| 8    | Master       | The Slave didn't send a long enough reply to the pairing ping                                                                                                               |
| 10   | Master       | The Slave sent a reply to the pairing ping that was too long                                                                                                                |
| 11   | Both         | Synchronisation failure, if this happens then there is something very wrong with the pairing system because it means pairing took too long but neither party detected this. |

### Sending Morse Code

If pairing succeeded, then the **Slave** display will go blank, and the **Master** display will show `?`. This is your prompt to enter Morse Code on the **Master**:

- Morse Code is entered by tapping Button <kbd>A</kbd> on the **Master** only.
- The Morse Code parser uses a fixed duration of one unit = 100ms (this can be changed by changing the value of the macro `MORSE_UNIT_MS` in `source/main.cpp`).
- A limited subset of International Morse Code is currently implemented. This comprises of upper-case `A-Z` and digits.
- As you enter Morse Code, the display of the **Master** will change; It will flash the middle pixel with every button press, and it will display the current character it _thinks_ you want to enter, based on what pulses you have entered so far. For example, if you were to enter `I` (dot dot), the display would briefly flash `E` as a single dot represents `E`, if you had entered a single dot and left it there, `E` would be detected and sent to the **Slave**.
- Once a letter space is encountered, the decoded letter is transmitted to the **Slave** over the _Line_. The **Slave** will briefly flash the letters sent to it on its own display. Due to limitations of the digital signalling protocol being used, there is some noticeable lag when doing this for one letter, however if lengthier text is entered this lag becomes no worse than that of user input.
- If you enter an invalid Morse Code sequence, or one that is not implemented by this software, the **Master** displays `!`.
- If the current sequence of Morse Codes you have entered is not valid but derivatives of it _might_ be, then `?` is displayed.

## State Diagrams

Here follow some state transition diagrams describing the pairing process from a signalling point of view:

```mermaid
stateDiagram
  state "Choose Role" as ChooseRole
    [*] --> ChooseRole
  ChooseRole --> PairMaster : Press Button A
  ChooseRole --> PairSlave : Press Button B
  state PairMaster {
    state "Begin Transmission Cycle as Master" as TransmissionCycle
    state "Try Pairing as Master" as TryPairMaster
    [*] --> TryPairMaster
    TryPairMaster --> TransmissionCycle : Successfully Paired
    TryPairMaster --> [*] : Failed to Pair
    TransmissionCycle --> [*] : Finished Transmitting
    TransmissionCycle --> TransmissionCycle : Transmission Session Open
  }
  PairMaster --> ChooseRole
  state PairSlave {
    state "Begin Transmission Cycle as Slave" as TransmissionCycle
    state "Try Pairing as Slave" as TryPairSlave
    [*] --> TryPairSlave
    TryPairSlave --> TransmissionCycle : Successfully Paired
    TryPairSlave --> [*] : Failed to Pair
    TransmissionCycle --> [*] : Finished Transmitting
    TransmissionCycle --> TransmissionCycle : Transmission Session Open
  }
  PairSlave --> ChooseRole
```

## Pairing from Master's Perspective

```mermaid
stateDiagram
  state "Awaiting Pair" as AwaitingPair
  [*] --> AwaitingPair
  state "No other Masters" as NoOtherMasters
  AwaitingPair --> NoOtherMasters : Line is LOW
  AwaitingPair --> [*] : Line is HIGH
  state "Holding Line High" as HoldingLineHigh
  NoOtherMasters --> HoldingLineHigh : Waited 500ms
  NoOtherMasters --> NoOtherMasters : Line is LOW
  NoOtherMasters --> [*] : Line is HIGH
  state "Awaiting Reply from Slave" as AwaitingReplyFromSlave
  HoldingLineHigh --> AwaitingReplyFromSlave : Waited 500 ms
  state "Received Slave Reply" as ReceivedSlaveReply
  AwaitingReplyFromSlave --> ReceivedSlaveReply : Line is HIGH
  AwaitingReplyFromSlave --> AwaitingReplyFromSlave : Line is LOW
  AwaitingReplyFromSlave --> [*] : Waited 100ms
  note right of AwaitingReplyFromSlave : Synchronise Clock at end of 500ms
  ReceivedSlaveReply --> [*] : Line is LOW
  ReceivedSlaveReply --> ReceivedSlaveReply : Line is HIGH
  state "Awaiting Slave Finish" as AwaitingSlaveFinish
  ReceivedSlaveReply --> AwaitingSlaveFinish : Waited 450ms
  state "Awaiting Synchronisation" as AwaitingSync
  AwaitingSlaveFinish --> AwaitingSync : Line is LOW
  AwaitingSlaveFinish --> AwaitingSlaveFinish : Line is HIGH
  AwaitingSlaveFinish --> [*] : Waited 100ms
  state "Synchronised and Ready for Transmission" as Ready
  AwaitingSync --> Ready : Waited for T+1250ms

```

## Pairing from Slave's Perspective

```mermaid
stateDiagram
  state "Awaiting Pair" as AwaitingPair
  [*] --> AwaitingPair
  state "Received Master Ping" as ReceivedMasterPing
  AwaitingPair --> ReceivedMasterPing : Line is HIGH
  AwaitingPair --> AwaitingPair : Line is LOW
  AwaitingPair --> [*] : Waited 10000ms
  state "Awaiting Master Finish" as AwaitingMasterFinish
  ReceivedMasterPing --> AwaitingMasterFinish : Waited 450ms
  ReceivedMasterPing --> ReceivedMasterPing : Line is HIGH
  ReceivedMasterPing --> [*] : Line is LOW
  AwaitingMasterFinish --> HoldingLineHigh : Line is LOW
  AwaitingMasterFinish --> AwaitingMasterFinish : Line is HIGH
  AwaitingMasterFinish --> [*] : Waited 100ms
  state "Holding Line High" as HoldingLineHigh
  note right of HoldingLineHigh : Synchronise Clock on falling edge
  state "Awaiting Synchronisation" as AwaitingSync
  HoldingLineHigh --> AwaitingSync : Waited 500ms
  state "Synchronised and Ready for Transmission" as Ready
  AwaitingSync --> Ready : Waited for T+1250ms
```

#### A brief word on Program Design

It's notable that this program is a lot more complicated than it needs to be to do what it does, this is because there was a lot more functionality planned for inclusion, including a custom 6-bit codepage with control characters, parity bits and some simple encryption using Diffie-Hellman for key exchange, but unfortunately I ran out of time due to the core signalling code taking a lot longer to complete than anticipated.

## Coding Convention

This project follows the **PEP 8 Style Guide for Python Code**, at least as much as a style guide for Python can be followed when writing C++, which is actually quite a bit!

## License

Licensed under the Mozilla Public License v2.0 (`MPL-2.0`).