/**
 * @file
 *
 * @brief Implementation of class ManchesterDecoder
 *
 * @author Joshua Saxby <Joshua2.Saxby@live.uwe.ac.uk>
 * @date 2020
 *
 * Student Number 18018052
 *
 * @copyright Copyright (C) Joshua Saxby 2020
 *
 * @copyright
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include <cstdint> // uint8_t

#include "ManchesterDecoder.hpp"


// SAXBOPHONE.COM is my domain, so I'm using the reverse form as my namespace
namespace com_saxbophone {
    ManchesterDecoder::ManchesterDecoder()
        : symbol_index(0) // the first symbol to read is definitely number 0
        {}

    bool ManchesterDecoder::input(bool state) {
        // guard condition --if the buffer is full, do not accept further input
        if (this->buffer.is_full()) return false;
        // otherwise, the buffer has further space to cache
        // every second received symbol is stored into the single byte buffer
        if (this->symbol_index % 2) {
            // {0..15} / 2 -> {0..7}        NOTE: big-endian
            this->current_byte |= (state << (7 - (this->symbol_index / 2)));
        }
        // if this is the last symbol, then store byte in buffer and clear byte
        if (this->symbol_index == 15) {
            this->buffer.add(this->current_byte);
            this->current_byte = 0x00u;
        }
        // always increment symbol index, wrap around at 16
        this->symbol_index = (this->symbol_index + 1) % 16;
        return true;
    }

    bool ManchesterDecoder::get_next_byte(uint8_t& destination) {
        // guard condition --check if buffer has content
        if (this->buffer.is_empty()) return false;
        // read out the first item in buffer and remove it
        destination = this->buffer[0];
        this->buffer.shrink_from_front(1);
        return true;
    }

    void ManchesterDecoder::clear() {
        this->buffer.clear();
        // also reset state variables
        this->symbol_index = 0;
        this->current_byte = 0x00u;
    }
};
