#ifndef COM_SAXBOPHONE_TRANSCEIVER_HPP
#define COM_SAXBOPHONE_TRANSCEIVER_HPP

/**
 * @file
 *
 * @brief A class providing a very basic trasceiver, that only knows how to pair
 * two micro:bits together and how to encode bytes sent to it into Manchester
 * Code.
 *
 * @details TODO ADD DESCRIPTION HERE
 *
 * @author Joshua Saxby <Joshua2.Saxby@live.uwe.ac.uk>
 * @date 2020
 *
 * Student Number 18018052
 *
 * @copyright Copyright (C) Joshua Saxby 2020
 *
 * @copyright
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#include <cstdint>

#include "Role.hpp"


// SAXBOPHONE.COM is my domain, so I'm using the reverse form as my namespace
namespace com_saxbophone {
    /**
     * @brief A basic class responsible only for low-level comms
     */
    class Transceiver {
    public:
        /**
         * @brief Used to allow synchronising both of a paired Master and Slave
         * @details Blocks the caller until a point in time a fixed duration
         * after the pairing synchronisation epoch is reached, if such point is
         * in the future. Otherwise, returns immediately.
         * @returns true if called between successful pairing and the
         * synchronisation epoch.
         * @returns false if called whilst not paired, or if the synchronisation
         * epoch is in the past
         */
        bool barrier();

        /**
         * @brief Attempts to send the given byte to the connected peer
         * @details This method is non-blocking
         * @returns true if this was done successfully
         * @returns false if this could not be done
         * @warn Bytes can only be sent once the Transceiver has been paired!
         * @warn Currently only the Master can send
         * @note This is a very low-level API with no receipt-confirmation, so
         * this method cannot tell you if the sent byte was received
         * successfully by the remote peer!
         */
        bool send(uint8_t byte);

        /**
         * @brief Closes down the connection and stops any currently active
         * transmission.
         */
        void close();
    };
};

#endif // include guard
