#ifndef COM_SAXBOPHONE_TRANSCEIVER_CONFIG_HPP
#define COM_SAXBOPHONE_TRANSCEIVER_CONFIG_HPP

/**
 * @file
 *
 * @brief POD type used for storing transceiver protocol config.
 * @note All config parameters are static, therefore this is the global config.
 *
 * @author Joshua Saxby <Joshua2.Saxby@live.uwe.ac.uk>
 * @date 2020
 *
 * Student Number 18018052
 *
 * @copyright Copyright (C) Joshua Saxby 2020
 *
 * @copyright
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */


// SAXBOPHONE.COM is my domain, so I'm using the reverse form as my namespace
namespace com_saxbophone {
    /**
     * @brief represents input pulses of Morse Code or digital signals.
     */
    struct TransceiverConfig {
        /**
         * @brief Pulses longer than this duration are ignored for decoding.
         * @details This is set to 200000µs (200ms)
         */
        const static unsigned long MAXIMUM_PULSE_DURATION = 120000;

        /**
         * @brief STOP/START bits are useful for demarcating when a transmission
         * begins and ends.
         * @details This is set to 600µs (6ms)
         */
        const static unsigned long STOP_START_BIT_DURATION = 6000;
        const static unsigned long STOP_START_BIT_DURATION_MS = 6;

        /**
         * @brief This is the duration of a standard raw bit on the signal line.
         * @details This is set to 24000µs (24ms)
         */
        const static unsigned long SIGNAL_BIT_DURATION = 42000;
        const static unsigned long SIGNAL_BIT_DURATION_MS = 42;

        /**
         * @brief This is the margin of error we allow for when deciding what
         * type a pulse is based on duration.
         * @note 1.0 = 100% and 0.0 = 0%
         * @details We currently allow a ±30% timing margin
         */
        constexpr const static double DURATION_MARGIN = 0.30;

        /**
         * @brief This is the total amount of time allocated for one transmission
         * @details This includes START/STOP bits, message bits and a bit of
         * silence time for good measure. This is set to 900ms.
         * @note This is the only duration in this struct to use ms for units
         */
        const static unsigned long TRANSMISSION_WINDOW = 900;
    };

};

#endif // include guard
