// XXX: this is a basic test program of the ManchesterEncoder class, debug only

#include <cassert>
#include <cstddef>
#include <cstdint>
#include <cstdio>

#include "../source/ManchesterDecoder.hpp"
#include "../source/ManchesterEncoder.hpp"


int main() {
    using namespace com_saxbophone;

    struct TestCase {
        uint8_t input; // byte being sent into the encoder
        bool output[16]; // sequence of HIGH/LOW states to be used as output
    };

    // test cases for the encoder
    TestCase test_cases[] = {
        { 0x00u, { 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, }, },
        { 0x73u, { 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, }, },
        { 0x54u, { 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, }, },
        { 0xACu, { 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, }, },
    };

    // test each test case
    for (TestCase t : test_cases) {
        ManchesterEncoder encoder;
        ManchesterDecoder decoder;
        encoder.input(t.input);
        printf("Input:   \t%x\n", t.input);
        printf("Expected:\t");
        for (size_t i = 0; i < 16; i++) {
            printf("%u", t.output[i]);
            fflush(stdout);
        }
        printf("\n");
        printf("Actual:  \t");
        for (size_t i = 0; i < 16; i++) {
            int state = 0;
            encoder.get_next_state(state);
            assert(state == t.output[i]);
            decoder.input(state);
            printf("%u", state);
            fflush(stdout);
        }
        printf("\n");
        uint8_t decoded = 0xff;
        decoder.get_next_byte(decoded);
        printf("Decoded: \t%x\n", decoded);
        assert(decoded == t.input);
        // draw a bar
        printf("================================\n");
    }

    // additional test cases of every byte
    printf("IN -> MANCHESTER CODE -> OUT\n");
    ManchesterEncoder encoder;
    ManchesterDecoder decoder;
    uint8_t b = 0x00u;
    do {
        printf("%02x -> ", b);
        encoder.input(b);
        for (uint8_t i = 0; i < 16; i++) {
            int symbol;
            encoder.get_next_state(symbol);
            printf("%d", symbol);
            decoder.input(symbol);
        }
        uint8_t output = 0xffu;
        decoder.get_next_byte(output);
        printf(" -> %02x\n", output);
        assert(output == b);
        b++;
    } while(b != 0x00u); // detect unsigned wrap-around

    return 0;
}
